<?php

namespace Intergo\RedisProxy;

use Illuminate\Cache\CacheManager;
use Illuminate\Redis\RedisManager;
use Illuminate\Support\ServiceProvider;
use Intergo\RedisProxy\Proxies\CacheProxy;
use Intergo\RedisProxy\Proxies\RedisProxy;

class RedisProxyServiceProvider extends ServiceProvider
{
    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        if (function_exists('config_path')) { // function not available and 'publish' not relevant in Lumen
            $this->publishes([__DIR__ . '/../config/redis-proxy.php' => config_path('redis-proxy.php')], 'config');
        }
    }

    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(__DIR__ . '/../config/redis-proxy.php', 'redis-proxy');

        $this->app->singleton(CacheProxy::class, function($app){
            $cache = new CacheManager($app);
            $configuredStores = config('redis-proxy.cache.stores');
            return new CacheProxy($cache, $configuredStores);
        });

        $this->app->singleton(RedisProxy::class, function($app){
            $redisconfig = config('database.redis');
            $redisClient = $redisconfig['client'] ?? 'predis';
            $redis = new RedisManager($app, $redisClient, $redisconfig);

            $configuredConnections = config('redis-proxy.redis.connections');
            return new RedisProxy($redis, $configuredConnections);
        });
    }
}
